///////////////////////////////////////////////////////////////////////////////////////////////////
//
// Theresa core library
// Copyright (C) 2001 Camilla Drefvenborg <elmindreda@home.se>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
///////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef SHARED_THSYSTEM_H
#define SHARED_THSYSTEM_H
///////////////////////////////////////////////////////////////////////////////////////////////////

interface IThSystem
{
// constructors
	virtual														~IThSystem(void);
// methods
	virtual bool											start(void) = 0;
	virtual void											stop(void) = 0;
	virtual void											pause(void) = 0;
	virtual void											resume(void) = 0;
// display methods
	virtual bool											openDisplay(unsigned int width, unsigned int height) = 0;
	virtual void											closeDisplay(void) = 0;
// music methods
	virtual bool											openMusic(const char* fileName) = 0;
	virtual void											closeMusic(void) = 0;
// attributes
	virtual bool											isStarted(void) = 0;
	virtual bool											isPaused(void) = 0;
	virtual float											getTotalTime(void) = 0;
	virtual float											getDeltaTime(void) = 0;
	virtual void											setTotalTime(float time) = 0;
	virtual IThDisplay*								getDisplay(void) = 0;
	virtual IThMusic*									getMusic(void) = 0;
// static methods
	static bool												create(void);
};

///////////////////////////////////////////////////////////////////////////////////////////////////

extern ThSingleton<IThSystem> System;

///////////////////////////////////////////////////////////////////////////////////////////////////
#endif /*SHARED_THSYSTEM_H*/
///////////////////////////////////////////////////////////////////////////////////////////////////
